<?php

/**
 * @file
 * Functions needed to query into Copyscape API.
 */

/**
 * Helper function to work with copyscape API.
 */
function copyscape_api_text_search_internet($text, $encoding = 'UTF-8', $full = '') {
  return copyscape_api_text_search($text, $encoding, $full);
}

/**
 * Processes the params and calls the copyscape_api_call() to execute search.
 *
 * @param string $text
 *   The text to search for.
 * @param string $encoding
 *   Text encoding, defaults to UTF-8.
 * @param int $full
 *   If not empty, it should be a value between 1 and 10.
 * @param string $operation
 *   Operation to be executed.
 *
 * @return array|bool
 *   The result of the executed search.
 *
 * @see http://www.copyscape.com/api-guide.php#text
 */
function copyscape_api_text_search($text, $encoding = 'UTF-8', $full = 0, $operation = 'csearch') {
  $params = array(
    'e' => $encoding,
  );

  if (!empty($full)) {
    $params['c'] = $full;
  }

  return copyscape_api_call($operation, $params, array(2 => array('result' => 'array')), $text);
}

/**
 * Builds the query string and executes the call against Copyscape API.
 *
 * @param string $operation
 *   The operation to be executed.
 * @param array $params
 *   Parameters to be sent.
 * @param array $xml_spec
 *   XML Spec to use.
 * @param array $post_data
 *   The fields to post to the request.
 *
 * @return array|bool
 *   The functions returns FALSE if no response was received
 *     or
 *   the parsed string as an array.
 */
function copyscape_api_call($operation, array $params = array(), array $xml_spec = array(), array $post_data = array()) {
  $copyscape_url = variable_get('copyscape_api_url', COPYSCAPE_DEFAULT_API_URL);
  $copyscape_username = variable_get('copyscape_api_username', '');
  $copyscape_api_key = variable_get('copyscape_api_key', '');

  // If the settings for accessing Copyscape were not done, exit.
  if (empty($copyscape_username) || empty($copyscape_api_key)) {
    return FALSE;
  }

  // Prepare the query array.
  $query = array(
    'u' => $copyscape_username,
    'k' => $copyscape_api_key,
    'o' => $operation,
  );

  // Merge with $params.
  $query += $params;

  $url = url($copyscape_url, array('query' => $query));

  $curl = curl_init();

  curl_setopt($curl, CURLOPT_URL, $url);
  curl_setopt($curl, CURLOPT_TIMEOUT, 30);
  curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
  curl_setopt($curl, CURLOPT_POST, isset($post_data));

  if (isset($post_data)) {
    curl_setopt($curl, CURLOPT_POSTFIELDS, $post_data);
  }

  $response = curl_exec($curl);
  curl_close($curl);

  if (drupal_strlen($response)) {
    return copyscape_read_xml($response, $xml_spec);
  }

  return FALSE;
}

/**
 * Parses the xml mark-up given as parameter.
 *
 * @param string $xml
 *   The XML mark-up to parse.
 * @param array $spec
 *   The XML spec to use.
 *
 * @return array|bool
 *   FALSE if $xml is not valid XML mark-up or
 *   Parsed XML as an array if $xml is valid.
 */
function copyscape_read_xml($xml, $spec = array()) {
  global $_copyscape_xml_data, $_copyscape_xml_depth, $_copyscape_xml_ref, $_copyscape_xml_spec;

  $_copyscape_xml_data = array();
  $_copyscape_xml_depth = 0;
  $_copyscape_xml_ref = array();
  $_copyscape_xml_spec = $spec;

  $parser = xml_parser_create();

  xml_set_element_handler($parser, 'copyscape_xml_start', 'copyscape_xml_end');
  xml_set_character_data_handler($parser, 'copyscape_xml_data');

  if (!xml_parse($parser, $xml, TRUE)) {
    return FALSE;
  }

  xml_parser_free($parser);

  return $_copyscape_xml_data;
}

/**
 * XML parser start handler to be used in xml_set_element_handler().
 *
 * @link http://php.net/manual/en/function.xml-set-element-handler.php
 */
function copyscape_xml_start($parser, $name, $attribs) {
  global $_copyscape_xml_data, $_copyscape_xml_depth, $_copyscape_xml_ref, $_copyscape_xml_spec;

  $_copyscape_xml_depth++;

  $name = drupal_strtolower($name);

  if ($_copyscape_xml_depth == 1) {
    $_copyscape_xml_ref[$_copyscape_xml_depth] = &$_copyscape_xml_data;
  }
  else {
    if (!is_array($_copyscape_xml_ref[$_copyscape_xml_depth - 1])) {
      $_copyscape_xml_ref[$_copyscape_xml_depth - 1] = array();
    }

    if (@$_copyscape_xml_spec[$_copyscape_xml_depth][$name] == 'array') {
      if (!is_array(@$_copyscape_xml_ref[$_copyscape_xml_depth - 1][$name])) {
        $_copyscape_xml_ref[$_copyscape_xml_depth - 1][$name] = array();
        $key = 0;
      }
      else {
        $key = 1 + max(array_keys($_copyscape_xml_ref[$_copyscape_xml_depth - 1][$name]));
      }

      $_copyscape_xml_ref[$_copyscape_xml_depth - 1][$name][$key] = '';
      $_copyscape_xml_ref[$_copyscape_xml_depth] = &$_copyscape_xml_ref[$_copyscape_xml_depth - 1][$name][$key];
    }
    else {
      $_copyscape_xml_ref[$_copyscape_xml_depth - 1][$name] = '';
      $_copyscape_xml_ref[$_copyscape_xml_depth] = &$_copyscape_xml_ref[$_copyscape_xml_depth - 1][$name];
    }
  }
}

/**
 * XML parser end handler to be used in xml_set_element_handler().
 *
 * @link http://php.net/manual/en/function.xml-set-element-handler.php
 */
function copyscape_xml_end($parser, $name) {
  global $_copyscape_xml_depth, $_copyscape_xml_ref;

  unset($_copyscape_xml_ref[$_copyscape_xml_depth]);

  $_copyscape_xml_depth--;
}

/**
 * XML parser character data handler function.
 *
 * @link http://php.net/manual/en/function.xml-set-character-data-handler.php
 */
function copyscape_xml_data($parser, $data) {
  global $_copyscape_xml_depth, $_copyscape_xml_ref;

  if (is_string($_copyscape_xml_ref[$_copyscape_xml_depth])) {
    $_copyscape_xml_ref[$_copyscape_xml_depth] .= $data;
  }
}
